@@ -8,68 +8,36 @@ import torch.optim.lr_scheduler as lr_scheduler
 import yaml
 from torch.utils.tensorboard import SummaryWriter
 
-import admm
-import test  # import test.py to get mAP after each epoch
-from admm import GradualWarmupScheduler
-from models import *
-from check_compression import test_sparsity
-from utils.datasets import *
-from utils.utils import *
-
-mixed_precision = True
-try:  # Mixed precision training https://github.com/NVIDIA/apex
-    from apex import amp
-except:
-    print('Apex recommended for faster mixed precision training: https://github.com/NVIDIA/apex')
-    mixed_precision = False  # not installed
-
-upperpath =  'ModelOpt/AutoYOLO/'
-
-wdir = upperpath+'weights' + os.sep  # weights dir
-last = wdir + 'last.pt'
-best = wdir + 'best.pt'
-results_file = 'results.txt'
+import ModelOpt.AutoYOLO.admm
+import ModelOpt.AutoYOLO.test  # import test.py to get mAP after each epoch
+from ModelOpt.AutoYOLO.admm import GradualWarmupScheduler
+from ModelOpt.AutoYOLO.models import *
+from ModelOpt.AutoYOLO.check_compression import test_sparsity
+from ModelOpt.AutoYOLO.utils.datasets import *
+from ModelOpt.AutoYOLO.utils.utils import *
 
 
-# Hyperparameters
-hyp = {'giou': 3.54,  # giou loss gain
-       'cls': 37.4,  # cls loss gain
-       'cls_pw': 1.0,  # cls BCELoss positive_weight
-       'obj': 64.3,  # obj loss gain (*=img_size/320 if img_size != 320)
-       'obj_pw': 1.0,  # obj BCELoss positive_weight
-       'iou_t': 0.20,  # iou training threshold
-       'lr0': 5E-3,  # initial learning rate (SGD=5E-3, Adam=5E-4)
-       'lrf': 0.0005,  # final learning rate (with cos scheduler)
-       'momentum': 0.937,  # SGD momentum
-       'weight_decay': 0.0005,  # optimizer weight decay
-       'fl_gamma': 0.0,  # focal loss gamma (efficientDet default is gamma=1.5)
-       'hsv_h': 0.0138,  # image HSV-Hue augmentation (fraction)
-       'hsv_s': 0.678,  # image HSV-Saturation augmentation (fraction)
-       'hsv_v': 0.36,  # image HSV-Value augmentation (fraction)
-       'degrees': 1.98 * 0,  # image rotation (+/- deg)
-       'translate': 0.05 * 0,  # image translation (+/- fraction)
-       'scale': 0.05 * 0,  # image scale (+/- gain)
-       'shear': 0.641 * 0}  # image shear (+/- deg)
-
-# Overwrite hyp with hyp*.txt (optional)
-f = glob.glob('hyp*.txt')
-if f:
-    print('Using %s' % f[0])
-    for k, v in zip(hyp.keys(), np.loadtxt(f[0])):
-        hyp[k] = v
-
-# Print focal loss if gamma > 0
-if hyp['fl_gamma']:
-    print('Using FocalLoss(gamma=%g)' % hyp['fl_gamma'])
 
 
 def train(hyp):
-    # batch_time = AverageMeter()
-    # data_time = AverageMeter()
-    # losses = AverageMeter()
+    global mixed_precision
+    try:  # Mixed precision training https://github.com/NVIDIA/apex
+        from apex import amp
+    except:
+        print('Apex recommended for faster mixed precision training: https://github.com/NVIDIA/apex')
+        mixed_precision = False  # not installed
+
+    wdir = 'weights' + os.sep  # weights dir
+    last = wdir + 'last.pt'
+    best = wdir + 'best.pt'
+    results_file = './results.txt'
+
+    import pkg_resources
+    PRUNE_CONFIG_PATH = pkg_resources.resource_filename('ModelOpt.AutoYOLO.train', 'prune_config/'+ opt.config_file + ".yaml")
+    DATA_CONFIG_PATH = pkg_resources.resource_filename('ModelOpt.AutoYOLO.train', opt.data)
 
     cfg = opt.cfg
-    data = opt.data
+    data = DATA_CONFIG_PATH
     epochs = opt.epochs  # 500200 batches at bs 64, 117263 images = 273 epochs
     batch_size = opt.batch_size
     accumulate = max(round(64 / batch_size), 1)  # accumulate n times before optimizer update (bs 64)
@@ -245,9 +213,10 @@ def train(hyp):
 
 
 
+
         for i in range(opt.rho_num):
             current_rho = initial_rho * 10 ** i
-            ADMM = admm.ADMM(model, file_name=upperpath+"prune_config/" + opt.config_file + ".yaml" , rho=current_rho)
+            ADMM = admm.ADMM(model, file_name=PRUNE_CONFIG_PATH , rho=current_rho)
             admm.admm_initialization(opt, ADMM=ADMM, model=model)  # intialize Z variable
 
             for epoch in range(start_epoch, epochs):  # epoch ------------------------------------------------------------------
@@ -417,7 +386,7 @@ def train(hyp):
             print("Saving model.")
             torch.save(
                 model.module.state_dict() if type(model) is nn.parallel.DistributedDataParallel else model.state_dict(),
-                "{}model_pruned/yolov4_{}_{}_{}.pt".format(upperpath,
+                "./weights/yolov4_{}_{}_{}.pt".format(
                     current_rho, opt.config_file, opt.sparsity_type))
 
         if not opt.evolve:
@@ -432,11 +401,11 @@ def train(hyp):
     """masked retrain"""
     """=============="""
     if opt.masked_retrain:
-        ADMM = admm.ADMM(model, file_name=upperpath+"prune_config/" + opt.config_file + ".yaml", rho=initial_rho)
+        ADMM = admm.ADMM(model, file_name=PRUNE_CONFIG_PATH, rho=initial_rho)
         if not opt.resume:
             # possible weights are '*.pt', 'yolov3-spp.pt', 'yolov3-tiny.pt' etc.
-            print("\n>_ Loading file: {}model_pruned/yolov4_{}_{}_{}.pt".format(upperpath, initial_rho * 10 ** (opt.rho_num - 1), opt.config_file, opt.sparsity_type))
-            chkpt = torch.load("{}model_pruned/yolov4_{}_{}_{}.pt".format(upperpath, initial_rho * 10 ** (opt.rho_num - 1), opt.config_file, opt.sparsity_type), map_location=device)
+            print("\n>_ Loading file: {}yolov4_{}_{}_{}.pt".format(wdir, initial_rho * 10 ** (opt.rho_num - 1), opt.config_file, opt.sparsity_type))
+            chkpt = torch.load("{}yolov4_{}_{}_{}.pt".format(wdir, initial_rho * 10 ** (opt.rho_num - 1), opt.config_file, opt.sparsity_type), map_location=device)
             # chkpt = torch.load(weights, map_location=device)
             # load model
             try:
@@ -475,7 +444,7 @@ def train(hyp):
         # Initialize distributed training
         if device.type != 'cpu' and torch.cuda.device_count() > 1 and torch.distributed.is_available():
             dist.init_process_group(backend='nccl',  # 'distributed backend'
-                                    init_method='tcp://127.0.0.1:9999',  # distributed training init method
+                                    init_method='tcp://127.0.0.1:9919',  # distributed training init method
                                     world_size=1,  # number of nodes for distributed training
                                     rank=0)  # distributed training node rank
             model = torch.nn.parallel.DistributedDataParallel(model, find_unused_parameters=True)
@@ -699,9 +668,151 @@ def train(hyp):
         # torch.cuda.empty_cache()
         return results
 
+def main_autorun(arg_list):
+    global mixed_precision
+    mixed_precision = True
+    try:  # Mixed precision training https://github.com/NVIDIA/apex
+        from apex import amp
+    except:
+        print('Apex recommended for faster mixed precision training: https://github.com/NVIDIA/apex')
+        mixed_precision = False  # not installed
 
 
+    # Hyperparameters
+    hyp = {'giou': 3.54,  # giou loss gain
+        'cls': 37.4,  # cls loss gain
+        'cls_pw': 1.0,  # cls BCELoss positive_weight
+        'obj': 64.3,  # obj loss gain (*=img_size/320 if img_size != 320)
+        'obj_pw': 1.0,  # obj BCELoss positive_weight
+        'iou_t': 0.20,  # iou training threshold
+        'lr0': 5E-3,  # initial learning rate (SGD=5E-3, Adam=5E-4)
+        'lrf': 0.0005,  # final learning rate (with cos scheduler)
+        'momentum': 0.937,  # SGD momentum
+        'weight_decay': 0.0005,  # optimizer weight decay
+        'fl_gamma': 0.0,  # focal loss gamma (efficientDet default is gamma=1.5)
+        'hsv_h': 0.0138,  # image HSV-Hue augmentation (fraction)
+        'hsv_s': 0.678,  # image HSV-Saturation augmentation (fraction)
+        'hsv_v': 0.36,  # image HSV-Value augmentation (fraction)
+        'degrees': 1.98 * 0,  # image rotation (+/- deg)
+        'translate': 0.05 * 0,  # image translation (+/- fraction)
+        'scale': 0.05 * 0,  # image scale (+/- gain)
+        'shear': 0.641 * 0}  # image shear (+/- deg)
+
+    # Overwrite hyp with hyp*.txt (optional)
+    f = glob.glob('hyp*.txt')
+    if f:
+        print('Using %s' % f[0])
+        for k, v in zip(hyp.keys(), np.loadtxt(f[0])):
+            hyp[k] = v
+
+    # Print focal loss if gamma > 0
+    if hyp['fl_gamma']:
+        print('Using FocalLoss(gamma=%g)' % hyp['fl_gamma'])
+    parser = argparse.ArgumentParser()
+    parser.add_argument('--epochs', type=int, default=280)  # 500200 batches at bs 16, 117263 COCO images = 273 epochs
+    parser.add_argument('--batch-size', type=int, default=64)  # effective bs = batch_size * accumulate = 16 * 4 = 64
+    parser.add_argument('--cfg', type=str, default='cfg/csdarknet53s-panet-spp.cfg', help='*.cfg path')
+    parser.add_argument('--data', type=str, default='data/coco2014.data', help='*.data path')
+    parser.add_argument('--multi-scale', action='store_true', help='adjust (67%% - 150%%) img_size every 10 batches')
+    parser.add_argument('--img-size', nargs='+', type=int, default=[256,416,320], help='[min_train, max-train, test]')
+    parser.add_argument('--rect', action='store_true', help='rectangular training')
+    parser.add_argument('--resume', action='store_true', help='resume training from last.pt')
+    parser.add_argument('--nosave', action='store_true', help='only save final checkpoint')
+    parser.add_argument('--notest', action='store_true', help='only test final epoch')
+    parser.add_argument('--evolve', action='store_true', help='evolve hyperparameters')
+    parser.add_argument('--bucket', type=str, default='', help='gsutil bucket')
+    parser.add_argument('--cache-images', action='store_true', help='cache images for faster training')
+    parser.add_argument('--weights', type=str, default='weights/yolov4dense.pt', help='initial weights path')
+    parser.add_argument('--name', default='', help='renames results.txt to results_name.txt if supplied')
+    parser.add_argument('--device', default='', help='device id (i.e. 0 or 0,1 or cpu)')
+    parser.add_argument('--adam', action='store_true', help='use adam optimizer')
+    parser.add_argument('--single-cls', action='store_true', help='train as single-class dataset')
+    parser.add_argument('--freeze-layers', action='store_true', help='Freeze non-output layers')  
+    parser.add_argument('--verbose', action='store_true', help='Freeze non-output layers')  
+
+    parser.add_argument('--admm', action='store_true', help='do admm process')
+    parser.add_argument('--masked-retrain', action='store_true', help='do masked-retrain process')
+    parser.add_argument('--rho', default=0.0001, help='rho val')
+    parser.add_argument('--rho-num', default=1, help='order of magnitude （from 0.0001 x 1e-1 to 0.0001 x 1e-4)')
+    parser.add_argument('--admm-epochs', default=6, help='rho val')
+    parser.add_argument('--sparsity-type', type=str, default='block-punched', help='order of magnitude （from 0.0001 x 1e-1 to 0.0001 x 1e-4)')
+    parser.add_argument('--config-file', type=str, default='config_csdarknet53pan_v14', help='prune ratio config file')
+    parser.add_argument('--combine-progressive', action='store_true', help='progressive pruning')
+
+    # parser.add_argument('--admm-file', type=str, default='admm', help='admm configuration file')
+
+
+    global opt
+    opt = parser.parse_args(arg_list)
+    
+    opt.lr = hyp['lr0']
+    # opt.mixup = True
+    # opt.alpha = 0.3
+    # opt.smooth = True
+    # opt.smooth_eps = 0.1
+
+
+    opt.weights = last if opt.resume and not opt.weights else opt.weights
+
+    # check_git_status()
+    # opt.cfg = check_file(opt.cfg)  # check file
+    # opt.data = check_file(opt.data)  # check file
+    print(opt)
+    opt.img_size.extend([opt.img_size[-1]] * (3 - len(opt.img_size)))  # extend to 3 sizes (min, max, test)
+    global device
+    device = torch_utils.select_device(opt.device, apex=mixed_precision, batch_size=opt.batch_size)
+    if device.type == 'cpu':
+        mixed_precision = False
+
+    global tb_writer
+    tb_writer = None
+    # print('Start Tensorboard with "tensorboard --logdir=runs", view at http://localhost:6006/')
+    # tb_writer = SummaryWriter(comment=opt.name)
+    train(hyp)  # train normally
+    # exit(0)
 if __name__ == '__main__':
+    global mixed_precision
+
+    mixed_precision = True
+    try:  # Mixed precision training https://github.com/NVIDIA/apex
+        from apex import amp
+    except:
+        print('Apex recommended for faster mixed precision training: https://github.com/NVIDIA/apex')
+        mixed_precision = False  # not installed
+
+
+
+    # Hyperparameters
+    hyp = {'giou': 3.54,  # giou loss gain
+        'cls': 37.4,  # cls loss gain
+        'cls_pw': 1.0,  # cls BCELoss positive_weight
+        'obj': 64.3,  # obj loss gain (*=img_size/320 if img_size != 320)
+        'obj_pw': 1.0,  # obj BCELoss positive_weight
+        'iou_t': 0.20,  # iou training threshold
+        'lr0': 5E-3,  # initial learning rate (SGD=5E-3, Adam=5E-4)
+        'lrf': 0.0005,  # final learning rate (with cos scheduler)
+        'momentum': 0.937,  # SGD momentum
+        'weight_decay': 0.0005,  # optimizer weight decay
+        'fl_gamma': 0.0,  # focal loss gamma (efficientDet default is gamma=1.5)
+        'hsv_h': 0.0138,  # image HSV-Hue augmentation (fraction)
+        'hsv_s': 0.678,  # image HSV-Saturation augmentation (fraction)
+        'hsv_v': 0.36,  # image HSV-Value augmentation (fraction)
+        'degrees': 1.98 * 0,  # image rotation (+/- deg)
+        'translate': 0.05 * 0,  # image translation (+/- fraction)
+        'scale': 0.05 * 0,  # image scale (+/- gain)
+        'shear': 0.641 * 0}  # image shear (+/- deg)
+
+    # Overwrite hyp with hyp*.txt (optional)
+    f = glob.glob('hyp*.txt')
+    if f:
+        print('Using %s' % f[0])
+        for k, v in zip(hyp.keys(), np.loadtxt(f[0])):
+            hyp[k] = v
+
+    # Print focal loss if gamma > 0
+    if hyp['fl_gamma']:
+        print('Using FocalLoss(gamma=%g)' % hyp['fl_gamma'])
+
     parser = argparse.ArgumentParser()
     parser.add_argument('--epochs', type=int, default=280)  # 500200 batches at bs 16, 117263 COCO images = 273 epochs
     parser.add_argument('--batch-size', type=int, default=64)  # effective bs = batch_size * accumulate = 16 * 4 = 64
@@ -735,7 +846,7 @@ if __name__ == '__main__':
 
     # parser.add_argument('--admm-file', type=str, default='admm', help='admm configuration file')
 
-
+    global opt
 
     opt = parser.parse_args()
     
@@ -753,12 +864,14 @@ if __name__ == '__main__':
     # opt.data = check_file(opt.data)  # check file
     print(opt)
     opt.img_size.extend([opt.img_size[-1]] * (3 - len(opt.img_size)))  # extend to 3 sizes (min, max, test)
+    
+    global device
     device = torch_utils.select_device(opt.device, apex=mixed_precision, batch_size=opt.batch_size)
     if device.type == 'cpu':
         mixed_precision = False
 
-
-    tb_writer = None
+    
+    # tb_writer = None
     # print('Start Tensorboard with "tensorboard --logdir=runs", view at http://localhost:6006/')
     # tb_writer = SummaryWriter(comment=opt.name)
     train(hyp)  # train normally
